// This is the part of MPFRCPP, the C++ interface for MPFR library.

// Copyright (C) 2006 Alexey Beshenov <bav.272304@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301 USA.

#ifndef __COMPARISON_CPP
#define __COMPARISON_CPP

//------------------------------------------------------------
// MPFRCPP COMPARISON FUNCTIONS
//------------------------------------------------------------

//------------------------------------------------------------
// Changelog:
// 2006-06-12 -- catching errors if comparison with NaN
// operand(s) called
//------------------------------------------------------------

//------------------------------------------------------------
// is...
//------------------------------------------------------------

bool is_nan (const MPFR& x)
    {
    if ( mpfr_nan_p(x.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


bool is_inf (const MPFR& x)
    {
    if ( mpfr_inf_p(x.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


bool is_number (const MPFR& x)
    {
    if ( mpfr_number_p(x.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


bool is_zero (const MPFR& x)
    {
    if ( mpfr_zero_p(x.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


bool is_unordered (const MPFR& x, const MPFR& y)
    {
    if ( mpfr_unordered_p(x.mpc_x, y.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


//------------------------------------------------------------
// Signum
//------------------------------------------------------------

int sgn (const MPFR& x)
    {
    int n = mpfr_sgn(x.mpc_x);
    if (is_nan (x)) { throw  MPFR::error(__FILE__, __LINE__,  MPFR::error::cmp_nan); }
    if ( n < 0 ) { return -1; }
    if ( n == 0 ) { return 0; }
    if ( n > 0 ) { return +1; }
    }


//------------------------------------------------------------
// Comparison
//------------------------------------------------------------

bool operator> (const MPFR& x, const MPFR& y)
    {
    if (is_unordered (x, y)) { throw  MPFR::error(__FILE__, __LINE__,  MPFR::error::cmp_nan); }
    if ( mpfr_greater_p(x.mpc_x, y.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


bool operator>= (const MPFR& x, const MPFR& y)
    {
    if (is_unordered (x, y)) { throw  MPFR::error(__FILE__, __LINE__,  MPFR::error::cmp_nan); }
    if ( mpfr_greaterequal_p(x.mpc_x, y.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


bool operator< (const MPFR& x, const MPFR& y)
    {
    if (is_unordered (x, y)) { throw  MPFR::error(__FILE__, __LINE__,  MPFR::error::cmp_nan); }
    if ( mpfr_less_p(x.mpc_x, y.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


bool operator<= (const MPFR& x, const MPFR& y)
    {
    if (is_unordered (x, y)) { throw MPFR::error(__FILE__, __LINE__,  MPFR::error::cmp_nan); }
    if ( mpfr_lessequal_p(x.mpc_x, y.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


bool operator== (const MPFR& x, const MPFR& y)
    {
    if (is_unordered (x, y)) { throw  MPFR::error(__FILE__, __LINE__,  MPFR::error::cmp_nan); }
    if ( mpfr_equal_p(x.mpc_x, y.mpc_x) == 0 ) { return false; }
    else { return true; }
    }


inline bool operator!= (const MPFR& x, const MPFR& y)
    {
    return !(x==y);
    }

#endif                                            // __COMPARISON_CPP
