// This is the part of MPFRCPP, the C++ interface for MPFR library.

// Copyright (C) 2006 Alexey Beshenov <bav.272304@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301 USA.

#ifndef __FLOAT_ROUND_STYLE_CPP
#define __FLOAT_ROUND_STYLE_CPP

//------------------------------------------------------------
// MPFRCPP mp_rnd_t / std::float_round_style CONVERTERS
//------------------------------------------------------------

// MPFR internal type mp_rnd_t does not conform to
// ISO/IEC 14882:1998, section 18.2.1.3, `Type float_round_style'.

// Internal functions frs2mprnd() mprnd2frs() should
// perform conversion.

// While designing MPFRCPP functions, use only std::float_round_style
// as defined at <limits>.

// This functions was protected because they are not needed to
// non-friends.

mp_rnd_t MPFR::frs2mprnd(const std::float_round_style r)
    {
    if (r == std::round_toward_zero) { return GMP_RNDZ; }
    if (r == std::round_to_nearest) { return GMP_RNDN; }
    if (r == std::round_toward_infinity) { return GMP_RNDU; }
    if (r == std::round_toward_neg_infinity) { return GMP_RNDD; }
    }


std::float_round_style MPFR::mprnd2frs(const mp_rnd_t r)
    {
    if (r == GMP_RNDZ) { return std::round_toward_zero; }
    if (r == GMP_RNDN) { return std::round_to_nearest; }
    if (r == GMP_RNDU) { return std::round_toward_infinity; }
    if (r == GMP_RNDD) { return std::round_toward_neg_infinity; }
    }

#endif                                            // __FLOAT_ROUND_STYLE_CPP
