// This is the part of MPFRCPP, the C++ interface for MPFR library.

// Copyright (C) 2006 Alexey Beshenov <bav.272304@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
// MA 02110-1301 USA.

#ifndef __SPECIAL_CPP
#define __SPECIAL_CPP

//------------------------------------------------------------
// MPFRCPP SPECIAL FUNCTIONS
//------------------------------------------------------------

//------------------------------------------------------------
// Exponential functions
//------------------------------------------------------------

MPFR exp (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_exp (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR expm1 (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_expm1 (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR exp10 (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_exp10 (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR exp2 (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_exp2 (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


//------------------------------------------------------------
// Logarithmic functions
//------------------------------------------------------------

MPFR log (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_log (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR log10 (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_log10 (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR log2 (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_log2 (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR log1p (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_log1p (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


//------------------------------------------------------------
// Trigonometric functions
//------------------------------------------------------------

MPFR sin (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_sin (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR cos (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_cos (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR sec (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_sec (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR csc (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_csc (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR tan (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_tan (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR cot (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_cot (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


//------------------------------------------------------------
// Inverse trigonometric functions
//------------------------------------------------------------

MPFR asin (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_asin (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR acos (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_acos (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR atan (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_atan (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR atan2 (const MPFR &x, const MPFR &y)
    {
    mp_prec_t prec; std::float_round_style rnd;
    if (x.round_mode() == y.round_mode()) { rnd = x.round_mode(); }
    else { rnd = MPFR::default_round_mode(); }
    MPFR z(x.prec(), rnd);
    mpfr_atan2 (z.mpc_x, x.mpc_x, y.mpc_x, MPFR::frs2mprnd(rnd));
    return z;
    }


//------------------------------------------------------------
// Hyperbolic functions
//------------------------------------------------------------

MPFR sinh (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_sinh (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR cosh (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_cosh (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR sech (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_sech (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR csch (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_csch (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR tanh (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_tanh (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR coth (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_coth (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


//------------------------------------------------------------
// Inverse hyperbolic functions
//------------------------------------------------------------

MPFR asinh (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_asin (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR acosh (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_acosh (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR atanh (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_atanh (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


//------------------------------------------------------------
// Special functions
//------------------------------------------------------------

MPFR erf (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_erf (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR erfc (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_erfc (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR eint (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_eint (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR gamma (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_gamma (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


MPFR lngamma (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_lngamma (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


inline MPFR beta (const MPFR &x, const MPFR &y) { return gamma(x)*gamma(y)/gamma(x+y); }

MPFR zeta (const MPFR &x)
    {
    MPFR y(x.prec(), x.round_mode());
    mpfr_zeta (y.mpc_x, x.mpc_x, x.mpc_round_mode);
    return y;
    }


//------------------------------------------------------------
// Misc.
//------------------------------------------------------------

MPFR agm (const MPFR &x, const MPFR &y)
    {
    mp_prec_t prec; std::float_round_style rnd;
    if (x.round_mode() == y.round_mode()) { rnd = x.round_mode(); }
    else { rnd = MPFR::default_round_mode(); }
    MPFR z(x.prec(), rnd);
    mpfr_agm (z.mpc_x, x.mpc_x, y.mpc_x, MPFR::frs2mprnd(rnd));
    return z;
    }


MPFR hypot (const MPFR &x, const MPFR &y)
    {
    mp_prec_t prec; std::float_round_style rnd;
    if (x.round_mode() == y.round_mode()) { rnd = x.round_mode(); }
    else { rnd = MPFR::default_round_mode(); }
    MPFR z(x.prec(), rnd);
    mpfr_hypot (z.mpc_x, x.mpc_x, y.mpc_x, MPFR::frs2mprnd(rnd));
    return z;
    }


MPFR factorial (const unsigned long int &x, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    MPFR y(prec, rnd);
    mpfr_fac_ui (y.mpc_x, x, MPFR::frs2mprnd(rnd));
    return y;
    }


inline MPFR factorial (const unsigned int &x, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode()) { return factorial(static_cast<unsigned long int>(x)); }

//------------------------------------------------------------
// Constants
//------------------------------------------------------------

MPFR const_catalan (const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode(), const bool clear_cache = false)
    {
    MPFR x(prec, rnd);
    mpfr_const_catalan (x.mpc_x, MPFR::frs2mprnd(rnd));
    if (clear_cache) { MPFR::cache::free(); }
    return x;
    }


MPFR const_euler (const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode(), const bool clear_cache = false)
    {
    MPFR x(prec, rnd);
    mpfr_const_euler (x.mpc_x, MPFR::frs2mprnd(rnd));
    if (clear_cache) { MPFR::cache::free(); }
    return x;
    }


MPFR const_log2 (const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode(), const bool clear_cache = false)
    {
    MPFR x(prec, rnd);
    mpfr_const_log2 (x.mpc_x, MPFR::frs2mprnd(rnd));
    if (clear_cache) { MPFR::cache::free(); }
    return x;
    }


MPFR const_pi (const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode(), const bool clear_cache = false)
    {
    MPFR x(prec, rnd);
    mpfr_const_pi (x.mpc_x, MPFR::frs2mprnd(rnd));
    if (clear_cache) { MPFR::cache::free(); }
    return x;
    }


//------------------------------------------------------------
// Random numbers
//------------------------------------------------------------

MPFR random2 (const mp_size_t size, const mp_exp_t exp, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    MPFR x(prec, rnd);
    mpfr_random2 (x.mpc_x, size, exp);
    return x;
    }


MPFR urandomb (gmp_randstate_t state, const mp_prec_t prec=MPFR::default_prec(), const std::float_round_style rnd=MPFR::default_round_mode())
    {
    MPFR x(prec, rnd);
    mpfr_urandomb (x.mpc_x, state);
    return x;
    }

#endif // __SPECIAL_CPP
